<?php

namespace Drupal\access_token_auth\Services;

use Drupal\access_token_auth\Exception\TokenGeneratorException;
use Drupal\access_token_auth\Model\Token;
use Drupal\access_token_auth\TokenGeneratorInterface;
use Drupal\access_token_auth\TokenManagerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Psr\Log\LoggerInterface;

/**
 * Manager service for tokens.
 */
class TokenManager implements TokenManagerInterface {

  /**
   * The default token manager service id.
   */
  const DEFAULT_SERVICE_ID = 'access_token_auth.default_token_manager';

  /**
   * The token generator service.
   *
   * @var \Drupal\access_token_auth\TokenGeneratorInterface
   */
  protected TokenGeneratorInterface $tokenGenerator;

  /**
   * Database connection.
   *
   * @var \Drupal\Core\Database\Connection
   *   Database connection.
   */
  protected Connection $connection;

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * The logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected LoggerInterface $logger;

  /**
   * TokenManager class constructor.
   *
   * @param \Drupal\access_token_auth\TokenGeneratorInterface $token_generator
   *   The token generator service.
   * @param \Drupal\Core\Database\Connection $connection
   *   Database connection.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger service.
   */
  public function __construct(
    TokenGeneratorInterface $token_generator,
    Connection $connection,
    ConfigFactoryInterface $config_factory,
    LoggerInterface $logger
  ) {
    $this->tokenGenerator = $token_generator;
    $this->connection = $connection;
    $this->configFactory = $config_factory;
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public function getReadableName(): string {
    return 'Default Token Manager';
  }

  /**
   * {@inheritdoc}
   */
  public function generateToken(int $user_id): ?Token {
    try {
      if (!$token = $this->getValidUserToken($user_id)) {
        $token = $this->tokenGenerator->generateToken($user_id);
        $this->insert($token);
        $this->logger->info('A token has been generated for user: @user_id', ['@user_id' => $user_id]);
      }

      return $token;
    }
    catch (TokenGeneratorException $exception) {
      $this->logger->error(
        'There was an error when trying to generate a token for user @user_id. \n Message: @message', [
          '@user_id' => $user_id,
          '@message' => $exception->getMessage(),
        ]
      );
    }

    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function validateToken(string $token): ?Token {
    $token = $this->getToken($token);

    if (!$token) {
      return NULL;
    }

    // Invalidate token (i.e. mark it as used) regardless of auth mode.
    $this->invalidateToken($token->getToken());
    $this->logger->info('A token has been validated for user: @user_id', ['@user_id' => $token->getUserId()]);

    return $token;
  }

  /**
   * Insert token into database.
   *
   * @param \Drupal\access_token_auth\Model\Token $token
   *   The token object.
   *
   * @return int
   *   The id in database after insert.
   *
   * @throws \Exception
   *   Database exception.
   */
  protected function insert(Token $token): int {
    return (int) $this->connection
      ->insert(Token::TABLE_NAME)
      ->fields([
        'token' => $token->getToken(),
        'user_id' => $token->getUserId(),
        'expire_at' => $token->getExpireAt(),
      ])
      ->execute();
  }

  /**
   * Get token from the database.
   *
   * @param string $token
   *   The token passed in to load.
   *
   * @return \Drupal\access_token_auth\Model\Token|null
   *   The token object or NULL if not found.
   */
  public function getToken(string $token): ?Token {
    $query = $this->connection
      ->select(Token::TABLE_NAME, 't')
      ->fields('t')
      ->condition('token', $token);

    if ($this->configFactory->get('access_token_auth.settings')->get('mode') === TOKEN::DEFAULT_MODE) {
      $query->condition('expire_at', time(), '>');
    }
    else {
      $query->condition('used_status', FALSE);
    }

    $token_raw = $query->execute()->fetchObject();

    if (empty($token_raw)) {
      return NULL;
    }

    return new Token($token_raw->token, $token_raw->user_id, $token_raw->expire_at, $token_raw->used_status);
  }

  /**
   * {@inheritdoc}
   */
  public function getValidUserToken(int $user_id): ?Token {
    $query = $this->connection
      ->select(Token::TABLE_NAME, 't')
      ->fields('t')
      ->condition('user_id', $user_id);

    if ($this->configFactory->get('access_token_auth.settings')->get('mode') === TOKEN::DEFAULT_MODE) {
      $query->condition('expire_at', time(), '>');
    }
    else {
      $query->condition('used_status', FALSE);
    }

    $token_raw = $query->execute()->fetchObject();

    if (empty($token_raw)) {
      return NULL;
    }

    return new Token($token_raw->token, $token_raw->user_id, $token_raw->expire_at, $token_raw->used_status);
  }

  /**
   * {@inheritdoc}
   */
  public function invalidateToken(string $token): void {
    $this->connection->update(Token::TABLE_NAME)
      ->fields([
        'used_status' => TRUE,
      ])
      ->condition('token', $token)
      ->execute();
  }

  /**
   * {@inheritdoc}
   */
  public function removeByToken(string $token): void {
    $this->connection->delete(Token::TABLE_NAME)
      ->condition('token', $token)
      ->execute();
  }

  /**
   * {@inheritdoc}
   */
  public function removeByUser(int $user_id): void {
    $this->connection->delete(Token::TABLE_NAME)
      ->condition('user_id', $user_id)
      ->execute();
  }

  /**
   * {@inheritdoc}
   */
  public function removeExpiredTokens(int $timestamp): void {
    if ($this->configFactory->get('access_token_auth.settings')->get('mode') === TOKEN::DEFAULT_MODE) {
      $this->connection->delete(Token::TABLE_NAME)
        ->condition('expire_at', $timestamp, '<')
        ->execute();
    }
    else {
      $this->connection->delete(Token::TABLE_NAME)
        ->condition('used_status', TRUE)
        ->execute();
    }
  }

}
